from __future__ import print_function

import sys
from sys import stderr, argv
from os.path import abspath, isfile
import sqlite3
from datetime import datetime as dt
from dateutil import parser as dt_parser

def _insert(val, sensor):
    """Returns INSERT query"""
    query = "INSERT INTO samples (id, value, sensor, timestamp) VALUES (NULL, %f, '%s', '%s');"
    return query % (val, sensor, dt.utcnow())

class FriskbyDao(object):

    def __init__(self, sql_path):
        """The sqlite db has a table called 'samples' with schema
        id, value, sensor, timestamp, uploaded
        """
        self._sql_path = abspath(sql_path)
        self.__init_sql()

    def get_path(self):
        return self._sql_path

    def __init_sql(self):
        if not isfile(self._sql_path):
            print('No database, constructing new: %s.' % self._sql_path)
            _id = '`id` INTEGER PRIMARY KEY'
            _val = '`value` FLOAT NOT NULL'
            _sen = '`sensor` TEXT NOT NULL'
            _date = '`timestamp` TEXT NOT NULL'
            _upl = '`uploaded` BOOL DEFAULT 0'
            _create = 'CREATE TABLE samples (%s, %s, %s, %s, %s);'
            schema = _create % (_id, _val, _sen, _date, _upl)
            conn = sqlite3.connect(self._sql_path)
            conn.execute(schema)
            conn.close()

    def get_num_rows(self, uploaded_status=None):
        """Gets num rows in sql storage.  If uploaded_status is set to True, we
        fetch number of rows that are marked uploaded, if uploaded_status is set
        to False, we fetch number of rows that are marked as not uploaded.
        """
        query = 'SELECT COUNT(uploaded) FROM samples'
        if uploaded_status is not None:
            if uploaded_status:
                query += ' WHERE uploaded'
            else:
                query += ' WHERE NOT uploaded'
        query += ';'
        conn = sqlite3.connect(self._sql_path)
        result = conn.execute(query)
        num = result.fetchone()[0]
        conn.close()
        return num


    def get_non_uploaded(self, limit=100):
        sub_q = "id, value, sensor, datetime(timestamp, 'localtime'), uploaded"
        query = 'SELECT %s FROM samples WHERE NOT `uploaded` LIMIT %d;'
        try:
            conn = sqlite3.connect(self._sql_path)
            result = conn.execute(query % (sub_q, limit))
            data = result.fetchall()
            conn.close()
            print('dao fetched %d rows of non-uploaded data' % len(data))
            sys.stdout.flush()
            for i in range(len(data)):
                id_, val_, sens_, dt_, upl_ = data[i]
                data[i] = id_, val_, sens_, dt_parser.parse(dt_), upl_
            return data
        except Exception as err:
            stderr.write('Error on reading data: %s.\n' % err)

    def persist_ts(self, data):
        ts_pm10, ts_pm25 = data
        q10 = _insert(ts_pm10.median(), 'PM10')
        q25 = _insert(ts_pm25.median(), 'PM25')
        try:
            conn = sqlite3.connect(self._sql_path)
            conn.execute(q10)
            conn.execute(q25)
            conn.commit()
            conn.close()
        except Exception as err:
            stderr.write('Error on persisting data: %s.\n' % err)
        print('Persisted data.')
        sys.stdout.flush()

    def mark_uploaded(self, data):
        print('dao marking ...')
        sys.stdout.flush()
        query = 'UPDATE samples SET uploaded=1 WHERE id=%s'
        try:
            conn = sqlite3.connect(self._sql_path)
            conn.execute('begin')
            for row in data:
                # id, value, sensor, timestamp, uploaded
                id_ = row[0]
                conn.execute(query % id_)
            conn.commit()
            conn.close()
        except Exception as err:
            stderr.write('Error on setting UPLOADED data! %s.\n' % err)


    def __repr__(self):
        try:
            num, num_up = self.get_num_rows(), self.get_num_rows(uploaded_status=True)
            fmt = 'FriskbyDao(num_rows = %s, non_uploaded = %s, path = %s)'
            return fmt % (num, num_up, self._sql_path)
        except:
            return 'FriskbyDao(path = %s)' % (self._sql_path)

if __name__ == '__main__':
    if len(argv) > 1:
        FriskbyDao(argv[1])



from __future__ import division

import serial
import time

class SDS011(object):
    msg_start  = 170
    msg_cmd    = 192
    msg_end    = 171

    sleep_time = 0.01

    device_usb = serial.Serial('/dev/ttyUSB0', baudrate=9600, stopbits=1, parity="N", timeout=2)
    device_ama = serial.Serial('/dev/ttyAMA0', baudrate=9600, stopbits=1, parity="N", timeout=2)

    def __init__(self, usb):
        if usb:
            self.device = SDS011.device_usb
        else:
            self.device = SDS011.device_ama

    def read(self):

        # Read in loop until message start: AAC0
        while True:
            s = self.device.read(1)
            if len(s) < 1:
                raise IOError('Device timed out in attempt to read a byte')
            if ord(s) == SDS011.msg_start:
                s = self.device.read(1)
                if ord(s) == SDS011.msg_cmd:
                    break
            time.sleep(SDS011.sleep_time)

        s = self.device.read(8)

        pm25hb = ord(s[0])
        pm25lb = ord(s[1])
        pm10hb = ord(s[2])
        pm10lb = ord(s[3])
        d5     = ord(s[4])
        d6     = ord(s[5])

        cs     = ord(s[6])
        tail   = ord(s[7])

        cs_expected = (pm25hb + pm25lb + pm10hb + pm10lb + d5 + d6) % 256
        if cs != cs_expected:
            raise Exception("Checksum test failed")

        if tail != SDS011.msg_end:
            raise Exception("Message was not correctly terminated?")

        pm25 = float(pm25hb + pm25lb*256)/10.0
        pm10 = float(pm10hb + pm10lb*256)/10.0

        return (pm10, pm25)

    @classmethod
    def is_mock(cls):
        return False



import os.path
import tempfile
import subprocess
import shutil
from git.repo.base import Repo

class GitModule(object):

    def clone(self, url):
        self.repo = Repo.clone_from(url, self.work_dir)


    def __init__(self, **kwargs):
        url = kwargs.get("url")
        local_path = kwargs.get("local_path")
        self.work_dir = None

        if url is None and local_path is None:
            raise ValueError("Must supply one of url and local_path")

        if url and local_path:
            raise ValueError("Can only supply one of url and local_path")

        if url:
            self.work_dir = tempfile.mkdtemp()
            self.clone(url)
        else:
            self.work_dir = None
            self.repo = Repo(local_path)
            origin = self.repo.remote(name="origin")
            origin.fetch()


    def __del__(self):
        if self.work_dir and os.path.isdir(self.work_dir):
            shutil.rmtree(self.work_dir)


    def getRoot(self):
        return self.repo.working_tree_dir

    def relPath(self, path):
        if path.startswith(self.getRoot()):
            N = len(self.getRoot()) + 1
            return path[N:]
        else:
            raise IOError("Not part of repo:%s ?!" % path)


    def absPath(self, path):
        repo_path = os.path.join(self.getRoot(), path)
        if os.path.exists(repo_path):
            return repo_path
        else:
            raise IOError("No such entry in repo:%s" % path)


    def checkout(self, ref):
        self.repo.git.checkout(ref)


    def getHeadSHA(self):
        commit = self.repo.commit("HEAD")
        return commit.hexsha

    def runTests(self, cmd):
        full_cmd = self.absPath(cmd)
        if not os.path.isfile(full_cmd):
            raise IOError("Is not an executable file:%s" % cmd)

        if not os.access(full_cmd, os.X_OK):
            raise OSError("File not executable: %s" % cmd)

        exit_code = subprocess.call(full_cmd)
        return exit_code == 0

    def install(self, target, files=(), directories=()):
        if os.path.exists(target):
            if not os.path.isdir(target):
                raise OSError("Target:%s already exists - and is not a directory" % target)
        else:
            os.makedirs(target)

        for dir_ in directories:
            repo_path = os.path.join(self.getRoot(), dir_)
            if not os.path.exists(repo_path):
                target_path = os.path.join(target, dir_)
                if not os.path.isdir(target_path):
                    os.makedirs(target_path)
                continue

            if os.path.dirname(dir_):
                target_path = os.path.join(target, os.path.dirname(dir_))
                if not os.path.isdir(target_path):
                    os.makedirs(target_path)

            for dirpath, _, filenames in os.walk(self.absPath(dir_)):
                target_path = os.path.join(target, self.relPath(dirpath))
                if not os.path.isdir(target_path):
                    os.makedirs(target_path)

                for file_ in filenames:
                    src_file = os.path.join(dirpath, file_)
                    target_file = os.path.join(target_path, file_)
                    shutil.copy(src_file, target_file)



        for file_ in files:
            target_file = os.path.join(target, file_)
            if os.path.dirname(file_):
                target_path = os.path.join(target, os.path.dirname(file_))
                if not os.path.isdir(target_path):
                    os.makedirs(target_path)

            shutil.copy(self.absPath(file_), target_file)
